import 'package:flutter/material.dart';
import 'package:puzzlegame/entities/moving_tile.dart';
import 'package:puzzlegame/provider/data.dart';
import 'package:puzzlegame/utils/game_utils.dart';

class Game {
  static void toggleDisplayTipImage(Data myProvider) {
    myProvider.updateIsTipImageDisplayed(!myProvider.displayTipImage);
  }

  static Container buildTilesetWidget(Data myProvider) {
    List<MovingTile> tiles = myProvider.tiles;

    Color borderColor = GameUtils.checkTilesetIsCleared(tiles) ? Colors.green : Colors.orange;
    int tileIndex = 0;

    Table tileset = Table(
      defaultColumnWidth: IntrinsicColumnWidth(),
      border: TableBorder.all(
        color: Colors.black,
        style: BorderStyle.solid,
        width: 2,
      ),
      children: [
        for (var row = 0; row < myProvider.tilesCount; row++)
          TableRow(
            children: [
              for (var col = 0; col < myProvider.tilesCount; col++)
                Column(children: [tiles[tileIndex++].widget(myProvider)]),
            ],
          ),
      ],
    );

    return Container(
      margin: EdgeInsets.all(8),
      padding: EdgeInsets.all(8),
      decoration: BoxDecoration(
        color: borderColor,
        borderRadius: BorderRadius.circular(8),
        border: Border.all(
          color: borderColor,
          width: 8,
        ),
      ),
      child: tileset,
    );
  }

  static Container buildTipWidget(Data myProvider) {
    return Container(
      margin: EdgeInsets.all(2),
      padding: EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              Column(
                children: [
                  Image(
                    image: AssetImage('assets/images/placeholder.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
              Column(
                children: [
                  TextButton(
                    child: Container(
                      decoration: BoxDecoration(
                        color: Colors.blue,
                        borderRadius: BorderRadius.circular(4),
                        border: Border.all(
                          color: Colors.blue,
                          width: 4,
                        ),
                      ),
                      child: Image(
                        image: AssetImage(
                          myProvider.displayTipImage
                              ? GameUtils.getImageAssetName(myProvider.selectedImage)
                              : 'assets/icons/tip_hidden.png',
                        ),
                        fit: BoxFit.contain,
                      ),
                    ),
                    onPressed: () => Game.toggleDisplayTipImage(myProvider),
                  ),
                ],
              ),
              Column(
                children: [
                  Image(
                    image: AssetImage('assets/images/placeholder.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
            ],
          ),
        ],
      ),
    );
  }

  static Container buildGameWidget(Data myProvider) {
    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          SizedBox(height: 8),
          buildTopIndicatorWidget(myProvider),
          SizedBox(height: 2),
          Expanded(
            child: buildGameBoard(myProvider),
          ),
          SizedBox(height: 2),
          Container(
            child: GameUtils.checkTilesetIsCleared(myProvider.tiles)
                ? Game.buildWinMessage(myProvider)
                : Game.buildTipWidget(myProvider),
          ),
        ],
      ),
    );
  }

  static Container buildGameBoard(Data myProvider) {
    return Container(
      margin: EdgeInsets.all(4),
      padding: EdgeInsets.all(4),
      child: Column(
        children: [
          buildTilesetWidget(myProvider),
        ],
      ),
    );
  }

  static Widget buildTopIndicatorWidget(Data myProvider) {
    return Table(
      children: [
        TableRow(
          children: [
            Column(children: [
              Text(
                myProvider.movesCount.toString(),
                style: TextStyle(
                  fontSize: 40,
                  fontWeight: FontWeight.w600,
                  color: Colors.black,
                ),
              ),
            ]),
            Column(children: []),
          ],
        ),
      ],
    );
  }

  static Container buildWinMessage(Data myProvider) {
    return Container(
      margin: EdgeInsets.all(2),
      padding: EdgeInsets.all(2),
      child: Table(
        defaultColumnWidth: IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              Column(
                children: [
                  Image(
                    image: AssetImage('assets/icons/game_win.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
              Column(
                children: [
                  TextButton(
                    child: Container(
                      child: Image(
                        image: AssetImage('assets/icons/button_back.png'),
                        fit: BoxFit.fill,
                      ),
                    ),
                    onPressed: () => GameUtils.resetGame(myProvider),
                  ),
                ],
              ),
              Column(
                children: [
                  Image(
                    image: AssetImage('assets/icons/game_win.png'),
                    fit: BoxFit.fill,
                  ),
                ],
              ),
            ],
          ),
        ],
      ),
    );
  }

  static Container buildShufflingIndicatorWidget() {
    return Container(
      child: Column(
        mainAxisSize: MainAxisSize.min,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Text(
            '⏳',
            style: TextStyle(
              fontSize: 60,
              fontWeight: FontWeight.w600,
              color: Colors.black,
            ),
          ),
          SizedBox(height: 20),
          CircularProgressIndicator(),
        ],
      ),
    );
  }
}
